package com.my.system.service.impl;

import com.my.common.core.domain.entity.SysUser;
import com.my.common.exception.ServiceException;
import com.my.common.utils.SecurityUtils;
import com.my.common.utils.StringUtils;
import com.my.system.mapper.SysUserMapper;
import com.my.system.service.ISysUserService;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import javax.validation.Validator;
import java.util.List;

/**
 * 用户 业务层处理
 * 
 * @author ruoyi
 */
@Service
public class SysUserServiceImpl implements ISysUserService
{
    private static final Logger log = LoggerFactory.getLogger(SysUserServiceImpl.class);

    @Autowired
    private SysUserMapper userMapper;

//    @Autowired
//    private SysRoleMapper roleMapper;
//
//    @Autowired
//    private SysPostMapper postMapper;
//
//    @Autowired
//    private SysUserRoleMapper userRoleMapper;
//
//    @Autowired
//    private SysUserPostMapper userPostMapper;

    @Autowired
    protected Validator validator;

    /**
     * 根据条件分页查询用户列表
     * 
     * @param user 用户信息
     * @return 用户信息集合信息
     */
    @Override
//    @DataScope(deptAlias = "d", userAlias = "u")
    public List<SysUser> selectUserList(SysUser user)
    {
        return userMapper.selectUserList(user);
    }

    /**
     * 通过用户名查询用户
     *
     * @param userName 用户名
     * @return 用户对象信息
     */
    @Override
    public List<SysUser> selectUserByUserName(String userName)
    {
        return userMapper.selectUserByUserName(userName);
    }

    /**
     * 通过用户ID查询用户
     *
     * @param userId 用户ID
     * @return 用户对象信息
     */
    @Override
    public SysUser selectUserById(Long userId)
    {
        return userMapper.selectUserById(userId);
    }
    @Override
    public List<SysUser> selectAllUser() {
        return userMapper.selectAllUser();
    }
    /**
     * 更新用户所属部门deptId
     */
    @Override
    public void updateUserDeptId(){
        userMapper.updateUserDeptId();
    }

    @Override
    public void updateDelFlag() {
        userMapper.updateDelFlag();
    }



    @Override
    public List<SysUser> selectUserByUserGuid(String userGuid) {
        return userMapper.selectUserByUserGuid(userGuid);
    }

    @Override
    public void insterH4aUser(SysUser user) {
        userMapper.insertUser(user);
    }

    @Override
    public void updateH4aUser(SysUser user) {
        userMapper.updateUser(user);
    }

    /**
     * 修改用户基本信息
     *
     * @param user 用户信息
     * @return 结果
     */
    @Override
    public int updateUserProfile(SysUser user)
    {
        return userMapper.updateUser(user);
    }


    /**
     * 修改用户类型
     * @return
     */
    public int updateUserType(String userId, String userType){
        return userMapper.updateUserType(userId, userType );
    }

    /**
     * 委托审批
     *  /** 委托时，1、他人委托给自己的权限不可委托出去，2、权限委托给了他人的用户不可接受他人委托
     * @param type 1-委托，2-取消委托
     * @return
     */
    @Override
    @Transactional
    public boolean deputeExamine(String type, Long userId) {
        Long selfId =  SecurityUtils.getLoginUser().getUserId();
        SysUser sysUser = selectUserById(selfId);

        if("2".equals(type)){ // 取消委托
            userId = sysUser.getDeputeUser();
            // 更新委托人信息（纪录委托给了谁）
            userMapper.updateDepute(selfId, null, null);
            // 更新接受委托人信息，
            userMapper.updateDepute(userId, null, null);
        } else {
            if("0".equals(sysUser.getUserType())){
                throw new ServiceException("用户无权限可委托！");
            }
            SysUser sysUser1 = selectUserById(userId);
            // 判断，如果接受委托的人自己的权限也委托出去了，则不可以接收别人的委托
            if(StringUtils.isEmpty(sysUser1.getDeputeType()) && sysUser1.getDeputeUser() != null){
                throw new ServiceException("接受委托的用户自己的权限已委托给他人，不可接受委托！");
            }
            // TODO 多人委托给同一用户

            // 更新委托人信息（纪录委托给了谁）
            userMapper.updateDepute(selfId, userId, null);
            // 更新接受委托人信息，
            userMapper.updateDepute(userId, selfId, sysUser.getUserType());
        }


        return true;
    }
    @Override
    public Long checkUserNameUnique(String userName)
    {
        SysUser info = userMapper.checkUserNameUnique(userName);
        if (StringUtils.isNotNull(info) )
        {
            return info.getUserId();
        }
        return null;
    }

    @Override
    public SysUser selectById(Long userId){
        return userMapper.selectById(userId);
    }


    @Override
    public int updateUserLoginFlag(String userGuid, long userId) {
        return userMapper.updateUserLoginFlag(userGuid, userId);
    }

//    /**
//     * 根据条件分页查询已分配用户角色列表
//     *
//     * @param user 用户信息
//     * @return 用户信息集合信息
//     */
//    @Override
//    @DataScope(deptAlias = "d", userAlias = "u")
//    public List<SysUser> selectAllocatedList(SysUser user)
//    {
//        return userMapper.selectAllocatedList(user);
//    }
//
//    /**
//     * 根据条件分页查询未分配用户角色列表
//     *
//     * @param user 用户信息
//     * @return 用户信息集合信息
//     */
//    @Override
//    @DataScope(deptAlias = "d", userAlias = "u")
//    public List<SysUser> selectUnallocatedList(SysUser user)
//    {
//        return userMapper.selectUnallocatedList(user);
//    }
//
//    /**
//     * 查询用户所属角色组
//     *
//     * @param userName 用户名
//     * @return 结果
//     */
//    @Override
//    public String selectUserRoleGroup(String userName)
//    {
//        List<SysRole> list = roleMapper.selectRolesByUserName(userName);
//        if (CollectionUtils.isEmpty(list))
//        {
//            return StringUtils.EMPTY;
//        }
//        return list.stream().map(SysRole::getRoleName).collect(Collectors.joining(","));
//    }
//
//    /**
//     * 查询用户所属岗位组
//     *
//     * @param userName 用户名
//     * @return 结果
//     */
//    @Override
//    public String selectUserPostGroup(String userName)
//    {
//        List<SysPost> list = postMapper.selectPostsByUserName(userName);
//        if (CollectionUtils.isEmpty(list))
//        {
//            return StringUtils.EMPTY;
//        }
//        return list.stream().map(SysPost::getPostName).collect(Collectors.joining(","));
//    }
//
//    /**
//     * 校验用户名称是否唯一
//     *
//     * @param user 用户信息
//     * @return 结果
//     */

//
//    /**
//     * 校验手机号码是否唯一
//     *
//     * @param user 用户信息
//     * @return
//     */
//    @Override
//    public boolean checkPhoneUnique(SysUser user)
//    {
//        Long userId = StringUtils.isNull(user.getUserId()) ? -1L : user.getUserId();
//        SysUser info = userMapper.checkPhoneUnique(user.getPhonenumber());
//        if (StringUtils.isNotNull(info) && info.getUserId().longValue() != userId.longValue())
//        {
//            return UserConstants.NOT_UNIQUE;
//        }
//        return UserConstants.UNIQUE;
//    }
//
//    /**
//     * 校验email是否唯一
//     *
//     * @param user 用户信息
//     * @return
//     */
//    @Override
//    public boolean checkEmailUnique(SysUser user)
//    {
//        Long userId = StringUtils.isNull(user.getUserId()) ? -1L : user.getUserId();
//        SysUser info = userMapper.checkEmailUnique(user.getEmail());
//        if (StringUtils.isNotNull(info) && info.getUserId().longValue() != userId.longValue())
//        {
//            return UserConstants.NOT_UNIQUE;
//        }
//        return UserConstants.UNIQUE;
//    }
//
//    /**
//     * 校验用户是否允许操作
//     *
//     * @param user 用户信息
//     */
//    @Override
//    public void checkUserAllowed(SysUser user)
//    {
//        if (StringUtils.isNotNull(user.getUserId()) && user.isAdmin())
//        {
//            throw new ServiceException("不允许操作超级管理员用户");
//        }
//    }
//
//    /**
//     * 校验用户是否有数据权限
//     *
//     * @param userId 用户id
//     */
//    @Override
//    public void checkUserDataScope(Long userId)
//    {
//        if (!SysUser.isAdmin(SecurityUtils.getUserId()))
//        {
//            SysUser user = new SysUser();
//            user.setUserId(userId);
//            List<SysUser> users = SpringUtils.getAopProxy(this).selectUserList(user);
//            if (StringUtils.isEmpty(users))
//            {
//                throw new ServiceException("没有权限访问用户数据！");
//            }
//        }
//    }
//
//    /**
//     * 新增保存用户信息
//     *
//     * @param user 用户信息
//     * @return 结果
//     */
//    @Override
//    @Transactional
//    public int insertUser(SysUser user)
//    {
//        // 新增用户信息
////        user.setUserId(new SnowflakeIdWorker(0, 0).nextId());
//        int rows = userMapper.insertUser(user);
//        // 新增用户岗位关联
//        insertUserPost(user);
//        // 新增用户与角色管理
//        insertUserRole(user);
//        return rows;
//    }
//
//    /**
//     * 注册用户信息
//     *
//     * @param user 用户信息
//     * @return 结果
//     */
//    @Override
//    public boolean registerUser(SysUser user)
//    {
//        return userMapper.insertUser(user) > 0;
//    }
//
//    /**
//     * 修改保存用户信息
//     *
//     * @param user 用户信息
//     * @return 结果
//     */
//    @Override
//    @Transactional
//    public int updateUser(SysUser user)
//    {
//        Long userId = user.getUserId();
//        // 删除用户与角色关联
//        userRoleMapper.deleteUserRoleByUserId(userId);
//        // 新增用户与角色管理
//        insertUserRole(user);
//        // 删除用户与岗位关联
//        userPostMapper.deleteUserPostByUserId(userId);
//        // 新增用户与岗位管理
//        insertUserPost(user);
//        return userMapper.updateUser(user);
//    }
//
//    /**
//     * 用户授权角色
//     *
//     * @param userId 用户ID
//     * @param roleIds 角色组
//     */
//    @Override
//    @Transactional
//    public void insertUserAuth(Long userId, Long[] roleIds)
//    {
//        userRoleMapper.deleteUserRoleByUserId(userId);
//        insertUserRole(userId, roleIds);
//    }
//
//    /**
//     * 修改用户状态
//     *
//     * @param user 用户信息
//     * @return 结果
//     */
//    @Override
//    public int updateUserStatus(SysUser user)
//    {
//        return userMapper.updateUser(user);
//    }
//
//
//    /**
//     * 修改用户头像
//     *
//     * @param userName 用户名
//     * @param avatar 头像地址
//     * @return 结果
//     */
//    @Override
//    public boolean updateUserAvatar(String userName, String avatar)
//    {
//        return userMapper.updateUserAvatar(userName, avatar) > 0;
//    }
//
//    /**
//     * 重置用户密码
//     *
//     * @param user 用户信息
//     * @return 结果
//     */
//    @Override
//    public int resetPwd(SysUser user)
//    {
//        return userMapper.updateUser(user);
//    }
//
//    /**
//     * 重置用户密码
//     *
//     * @param userName 用户名
//     * @param password 密码
//     * @return 结果
//     */
//    @Override
//    public int resetUserPwd(String userName, String password)
//    {
//        return userMapper.resetUserPwd(userName, password);
//    }
//
//    /**
//     * 新增用户角色信息
//     *
//     * @param user 用户对象
//     */
//    public void insertUserRole(SysUser user)
//    {
//        this.insertUserRole(user.getUserId(), user.getRoleIds());
//    }
//
//    /**
//     * 新增用户岗位信息
//     *
//     * @param user 用户对象
//     */
//    public void insertUserPost(SysUser user)
//    {
//        Long[] posts = user.getPostIds();
//        if (StringUtils.isNotEmpty(posts))
//        {
//            // 新增用户与岗位管理
//            List<SysUserPost> list = new ArrayList<SysUserPost>(posts.length);
//            for (Long postId : posts)
//            {
//                SysUserPost up = new SysUserPost();
//                up.setUserId(user.getUserId());
//                up.setPostId(postId);
//                list.add(up);
//            }
//            userPostMapper.batchUserPost(list);
//        }
//    }
//
//    /**
//     * 新增用户角色信息
//     *
//     * @param userId 用户ID
//     * @param roleIds 角色组
//     */
//    public void insertUserRole(Long userId, Long[] roleIds)
//    {
//        if (StringUtils.isNotEmpty(roleIds))
//        {
//            // 新增用户与角色管理
//            List<SysUserRole> list = new ArrayList<SysUserRole>(roleIds.length);
//            for (Long roleId : roleIds)
//            {
//                SysUserRole ur = new SysUserRole();
//                ur.setUserId(userId);
//                ur.setRoleId(roleId);
//                list.add(ur);
//            }
//            userRoleMapper.batchUserRole(list);
//        }
//    }
//
//    /**
//     * 通过用户ID删除用户
//     *
//     * @param userId 用户ID
//     * @return 结果
//     */
//    @Override
//    @Transactional
//    public int deleteUserById(Long userId)
//    {
//        // 删除用户与角色关联
//        userRoleMapper.deleteUserRoleByUserId(userId);
//        // 删除用户与岗位表
//        userPostMapper.deleteUserPostByUserId(userId);
//        return userMapper.deleteUserById(userId);
//    }
//
//    /**
//     * 批量删除用户信息
//     *
//     * @param userIds 需要删除的用户ID
//     * @return 结果
//     */
//    @Override
//    @Transactional
//    public int deleteUserByIds(Long[] userIds)
//    {
//        for (Long userId : userIds)
//        {
//            checkUserAllowed(new SysUser(userId));
//            checkUserDataScope(userId);
//        }
//        // 删除用户与角色关联
//        userRoleMapper.deleteUserRole(userIds);
//        // 删除用户与岗位关联
//        userPostMapper.deleteUserPost(userIds);
//        return userMapper.deleteUserByIds(userIds);
//    }
//


}
